#ifndef _VX_CONTEXT_H
#define _VX_CONTEXT_H

/*
	We may have a different domainname and nodename for each security
	context. By default, a security context share the same as its
	parent, potentially the information in system_utsname
*/
#define VX_INFO_LOCK		1	/* Can't request a new vx_id */
#define VX_INFO_SCHED		2	/* All process in the vx_id */
					/* Contribute to the schedular */
#define VX_INFO_NPROC		4	/* Limit number of processes in a context */
#define VX_INFO_PRIVATE		8	/* Noone can join this security context */
#define VX_INFO_INIT		16	/* This process wants to become the */
					/* logical process 1 of the security */
					/* context */
#define VX_INFO_HIDEINFO	32	/* Hide some information in /proc */
#define VX_INFO_ULIMIT		64	/* Use ulimit of the current process */
					/* to become the global limits */
					/* of the context */
	
#define MAX_S_CONTEXT 65535	/* Arbitrary limit */
#define MIN_D_CONTEXT 49152	/* dynamic contexts start here */

#define NB_S_CONTEXT	16

#define NB_IPV4ROOT	16

#include <asm/resource.h>
#include <asm/atomic.h>

struct context_info {
	atomic_t refcount;
	short int vx_id[NB_S_CONTEXT];/* root is allowed to switch the current */
				/* security context using any in this table */
	unsigned long rlim[RLIM_NLIMITS];	/* Per context limit */
	atomic_t res[RLIM_NLIMITS];	/* Current value */
	struct proc_dir_entry *procent;
	char nodename[65];
	char domainname[65];
	int flags;		/* VX_INFO_xxx */
	atomic_t ticks;		/* Number of ticks used by all process */
				/* in the vx_id */
	int initpid;		/* PID of the logical process 1 of the */
				/* of the context */
	int nr_threads;
	unsigned long total_forks;
	unsigned int bias_cswtch;
	long bias_jiffies;
	long bias_idle;
	void *data1;
	void *data2;
	void *data3;
	void *data4;
};

struct iproot_info {
	unsigned long mark;	/* Special signature for debugging */
	atomic_t refcount;
	int nbipv4;
	__u32 ipv4[NB_IPV4ROOT];/* Process can only bind to these IPs */
				/* The first one is used to connect */
				/* and for bind any service */
				/* The other must be used explicity when */
				/* binding */
	__u32 mask[NB_IPV4ROOT];/* Netmask for each ipv4 */
				/* Used to select the proper source address */
				/* for sockets */
	__u32 v4_bcast;	/* Broadcast address used to receive UDP packets */
};


#define	VX_ADMIN	0x0001
#define	VX_WATCH	0x0002

#define	VX_IDENT	0x0010
#define	VX_EQUIV	0x0020
#define	VX_PARENT	0x0040
#define	VX_CHILD	0x0080

#define	VX_ARG_MASK	0x00F0

#include <asm/current.h>

/* required to resolve recursive dependancies */
#define vx_check(c,m)	__vx_check(current->vx_id,c,m)

#define vx_weak_check(c,m)     ((m) ? vx_check(c,m) : 1)

/*
 * check current context for ADMIN/WATCH and
 * optionally agains supplied argument
 */
static inline int __vx_check(int cctx, int ctx, unsigned int mode)
{
	if (mode & VX_ARG_MASK) {
		if ((mode & VX_IDENT) && (ctx == cctx))
			return 1;
		if ((mode & VX_EQUIV) && (ctx == cctx))
			return 1;
	}
	return (((mode & VX_ADMIN) && (cctx == 0)) ||
		((mode & VX_WATCH) && (cctx == 1)));
}


void vx_assign_info(struct task_struct *);
void vx_release_info(struct task_struct *);

void vx_assign_ip_info(struct iproot_info *);
void vx_release_ip_info(struct iproot_info *);

int vc_new_s_context(uint32_t, void *);
int vc_set_ipv4root(uint32_t, void *);

/* procfs ioctls */

#define	FIOC_GETXFLG	_IOR('x', 5, long)
#define	FIOC_SETXFLG	_IOW('x', 6, long)

#endif
